//$Log: capiinfo.cpp,v $
//Revision 1.4  2005/01/30 23:55:40  tubsmaster
//- log file gets flushed every start
//- check if /dev/capi20 is writeable and readable
//
//Revision 1.3  2005/01/30 16:59:59  tubsmaster
//now we have ldap support for addressresolution
//

static const char capiinfo_cpp[] = "$Id: capiinfo.cpp,v 1.4 2005/01/30 23:55:40 tubsmaster Exp $";

#include "capiinfo.h"
#include <iostream>
/** 
* Class destructor.
* releases capi app from kernel
*/
capiInfo::~capiInfo()
{
    if (capi20_release(m_wApplId) != CapiNoError)
	throw capiError("release failed");
}

/** 
* Class constructor.
* Lets the Fritz!card listen on the S0-bus for incomming calls.
*/
capiInfo::capiInfo()
{

    m_wApplId = 1;
    m_wIntNr = 1;

    //check for rw before capi functions
    FILE *fp = fopen("/dev/capi20","w+");
    if (!fp)
      throw capiError("FATAL: /dev/capi20 is not readable or writeable!");
    fclose(fp);
    
    if (capi20_isinstalled() != CapiNoError)
	throw capiError("No Capi Installed");
    if (capi20_register(1, 2, 128, (unsigned int *) &m_wApplId) != CapiNoError)
	throw capiError("register failed");

    LISTEN_REQ(&cmsg, m_wApplId, m_wIntNr++, 0x1	//_cdword adr
	       , 0x0		//_cdword InfoMask
	       , 0x1		//_cdword CIPmask
	       , 0x0		//_cdword CIPmask2
	       , NULL		//_cstruct CallingPartyNumber
	       , NULL);		//_cstruct CallingPartySubaddress
 //   if (CAPI_PUT_CMSG(&cmsg) != CapiNoError)
//	throw capiError("cant put adr1 in listenmode");

    LISTEN_REQ(&cmsg, m_wApplId, m_wIntNr++, 0x2	//_cdword adr
	       , 0x0		//_cdword InfoMask
	       , 0x1		//_cdword CIPmask
	       , 0x0		//_cdword CIPmask2
	       , NULL		//_cstruct CallingPartyNumber
	       , NULL);		//_cstruct CallingPartySubaddress
  //  if (CAPI_PUT_CMSG(&cmsg) != CapiNoError)
//	throw capiError("cant put adr2 in listenmode");

    //assume normal isdn conroller -> 2 adr -> 1 channel per adr
    //todo: read information from capi_getprofile()

}

/** 
* does a blocked wait() for an incoming call (with timeout).
* Sets the card to wait for a message and if the message is an incomming call, read the number and return it.
* @param sec timeout value
* @return returns the incoming number and returns it as std::string. If timeout return NULL as std::string.
*/
std::string capiInfo::waitForCall(int sec)
{
    #ifdef DEBUG
    std::cout << "Wait for call..." << std::endl;
    #endif

    struct timeval mytime;
    if (sec == 0)
	capi20_waitformessage(m_wApplId, NULL);
    else
    {
	mytime.tv_sec = sec;
	mytime.tv_usec = 0;
	capi20_waitformessage(m_wApplId, &mytime);
    }

    capi20_get_cmsg(&cmsg, m_wApplId);


    #ifdef DEBUG
    printf("Command: %x \nSubcommand: %x\n", cmsg.Command , cmsg.Subcommand) ;
    #endif

    if (cmsg.Command == 0x2 && cmsg.Subcommand == 0x82)	//CONNECT_IND
    {
        #ifdef DEBUG
        std::cout << "incomming call from :" << cmsg.CallingPartyNumber << std::endl;
        #endif
	std::string callingPartyNumber((char *) cmsg.CallingPartyNumber);
        CONNECT_RESP(&cmsg, m_wApplId, m_wIntNr++, cmsg.adr.adrController, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0);
	if (CAPI_PUT_CMSG(&cmsg) != CapiNoError)
	    throw capiError("CONNECT_RESP failed...");
	//cut header from number (2 chars) if there is a number - leading 0 ist  missing
	// if no number is given - only 1 char exists!!
	if (callingPartyNumber.size() == 1)
	    return std::string("unknown");	//no number
	return (callingPartyNumber.substr(3)).insert(0, std::string("0"));
    }


    if (cmsg.Command == 0x4 && cmsg.Subcommand == 0x82)	//DISCONNECT_IND
    {
	DISCONNECT_RESP(&cmsg, m_wApplId, m_wIntNr++, cmsg.adr.adrController);
	if (CAPI_PUT_CMSG(&cmsg) != CapiNoError)
	    throw capiError("DISCONNECT_RESP failed...");
    }
    return std::string("NULL");


}
