//$Log: capiisdnmon.cpp,v $
//Revision 1.5  2005/01/30 23:55:40  tubsmaster
//- log file gets flushed every start
//- check if /dev/capi20 is writeable and readable
//
//Revision 1.4  2005/01/30 16:59:59  tubsmaster
//now we have ldap support for addressresolution
//

static const char capiisdnmon_cpp[] = "$Id: capiisdnmon.cpp,v 1.5 2005/01/30 23:55:40 tubsmaster Exp $";

#include "capiisdnmon.h"

#include "gtkdocklet.h"
#include <pwd.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <fstream>
#include <sstream>
#include <iostream>

using namespace std;

/**
* Class constructor.
* initializes all program parts.
* <ul>
* <li>Logger: handles logging stuff</li>
* <li>capiIsdnMonHist: History File for capiIsdnMon</li>
* <li>capiThread: the thread, which handles capi stuff</li>
* <li>capiIsdnMonConfig: the config stuff from capiIsdnMon.ini</li>
* <li>ldapAdressBook: name resolving stuff from evo Adressbook file</li>
* </ul>
* @param argc Amount of parameters.
* @author T. Gebhard
*/
capiIsdnMon::capiIsdnMon()
{
    struct passwd *PasswdEntry;
    PasswdEntry = getpwuid(getuid());

    //does ~/.capiIsdnMon exist and is writeable?
    fullCapiIsdnMonPath = std::string(PasswdEntry->pw_dir) + std::string("/.capiIsdnMon");
    DIR *dir = opendir(fullCapiIsdnMonPath.c_str());
    if (dir == NULL)
    {
	if (!mkdir(fullCapiIsdnMonPath.c_str(), 0755) == 0)
	{
	    std::cerr << "Could not open or create directory: '" << fullCapiIsdnMonPath << "'" << std::endl;
	    fullCapiIsdnMonPath = "invalid";
	}
    }

    //create logger
    capiIsdnMonLogger = new Logger(string(string(PasswdEntry->pw_dir) + string("/.capiIsdnMon") + string("/capiIsdnMon.log")));
    capiIsdnMonLogger->addToLog("Startup....");

    //reads config file, if exist
    _capiIsdnMonConfig = new capiIsdnMonConfig(fullCapiIsdnMonPath + string("/capiIsdnMon.ini"));
    try
    {
        _capiIsdnMonConfig->readConfigFile();
    }
    catch (std::string err_msg)
    {
        capiIsdnMonLogger->addToLog(err_msg);
    }
    
    //start capiThread - application quits on error
    _capiThread = new capiThread(this);
    _capiThread->run();

    //wait one sec for capi stuff
    sleep(1);

    //open ldap for addressresolution if configured
    _ldapAddressServer = NULL;
    if (_capiIsdnMonConfig->getWantLdap() == "true")
    {
	initLdap();
    }

    //open history file
    std::string HistFile("/capiIsdnMon.hist");
    HistFile = fullCapiIsdnMonPath + HistFile;
    if (!myCapiIsdnMonHist.prepareLogFile(HistFile))
    {
	capiIsdnMonLogger->addToLog(string("Error opening HistoryFile: ") + HistFile);
	haveValidHistFile = false;
    } else {
	haveValidHistFile = true;
	capiIsdnMonLogger->addToLog(string("History File opened successfully"));
    }

    //xosd init
    _xosdDisplay = new xosdDisplay();

    //gtk stuff - its c code
    gtk_docklet_init(this);
    gtk_docklet_load();
    gtk_timeout_add(2000,(GtkFunction) gtk_docklet_toggleIcon, NULL);
}

/**
* Deconstructor removes docklet.
*/
capiIsdnMon::~capiIsdnMon()
{
  gtk_docklet_unload();
}


/**
* Does a clean application shutdown.
*/
void capiIsdnMon::stopThreadAndQuit()
{
    _capiThread->stopThread();
    capiIsdnMonLogger->addToLog(string("initiating application shutdown."));
    _capiThread->waitForThreadEnd();    
    capiIsdnMonLogger->addToLog("application shutdown complete.");
    exit(0);

}

/**
* Shows a number on Screen and writes event to log.
* @param number the telephonenumber
*/
void capiIsdnMon::showNumber(std::string number)
{
    gdk_threads_enter ();
    gtk_docklet_toggleIconTrigger(2);
    gdk_threads_leave ();
    capiIsdnMonLogger->addToLog(string("Got Number: ") + number);
    std::string osdText = number;
    std::string Name;
    try
    {
	if (osdText != "unknown")
	{
	    if (_ldapAddressServer)
	    {
              Name = _ldapAddressServer->resolveNameForNumber(number);
	      osdText = osdText + ": " + Name;
	    }
        }
    } catch (std::string err_msg)
    {
        capiIsdnMonLogger->addToLog(err_msg);  
	//ldap error - delete it
	delete _ldapAddressServer;
	_ldapAddressServer = NULL;
    }
    capiIsdnMonLogger->addToLog(string("XOSD - showing: ") + osdText);
    _xosdDisplay->showLine(osdText);

    time_t ctime;
    time(&ctime);
    struct tm myTm;
    localtime_r(&ctime, &myTm);
    char logline[200];
    sprintf(logline, "%d-%02d-%02d %02d:%02d:%02d %16s    %s",
	    int (myTm.tm_year + 1900), myTm.tm_mon, myTm.tm_mday, myTm.tm_hour, myTm.tm_min, myTm.tm_sec, number.c_str(), Name.c_str());

    addRecent(std::string(logline));
    if (haveValidHistFile)
    {
	if (!myCapiIsdnMonHist.addCall(number, Name))
	    haveValidHistFile = false;
    }
}


/*
* Checks if Logfile is writeable etc...
* @param File thie Historyfilename with Path
* @return false = error opening historyfile
*/
bool capiIsdnMonHist::prepareLogFile(std::string File)
{
    ofstream filestr;
    filestr.open(File.c_str(), ofstream::out | ofstream::app);
    if (!filestr.good())
    {
	HistFile = "invalid";
	filestr.close();
	return false;
    } else
    {
	HistFile = File;
	filestr.close();
	return true;
    }
}

/*
* adds a Line to Historyfile, if possible
* @param Number the telephonenumber
* @param Caller the corresponding name
* @return true = successfully written to historyfile 
*/
bool capiIsdnMonHist::addCall(std::string Number, std::string Caller)
{
    time_t ctime;
    time(&ctime);
    struct tm myTm;
    localtime_r(&ctime, &myTm);

    char logline[200];
    sprintf(logline, "%d-%02d-%02d %02d:%02d:%02d %16s    %s",
	    int (myTm.tm_year + 1900), myTm.tm_mon, myTm.tm_mday, myTm.tm_hour, myTm.tm_min, myTm.tm_sec, Number.c_str(), Caller.c_str());

    fstream filestr;
    filestr.open(HistFile.c_str(), fstream::out | fstream::app);
    if (!filestr.good())
    {
	filestr.close();
	return false;
    }
    filestr << logline << std::endl;
    filestr.close();
    return true;
}

/*
* writes current ldap config into capiIsdnMon.ini
* @return true = successfully written 
*/
bool capiIsdnMonConfig::writeConfigToFile()
{
    ofstream filestr;
    filestr.open(_configFile.c_str());
    if (!filestr.good())
    {
        filestr.close();
        throw "Error opening config file";
    }
    else
    {
        string configString = string("activateLdap = ") + _wantLdap + 
                              string("\nldapHost = ") + _ldapHostname + 
                              string("\nldapPort = ") + _ldapPort + 
                              string("\nldapBase = ") + _ldapBase;
        filestr << configString << endl;
        filestr.close();
	chmod(_configFile.c_str(), S_IRUSR|S_IWUSR);
        return true;
    }
}

/*
* reads ldap config from capiIsdnMon.ini
* @return true = successfully read 
*/
bool capiIsdnMonConfig::readConfigFile()
{
    ifstream filestr;
    filestr.open(_configFile.c_str());
    if (!filestr.good())
    {
        filestr.close();
        throw string("Cant open config file(does not exist?)");
    }
    else
    {
        string tmpStr;
	int i;
        while (getline(filestr,tmpStr))
        {
	    while ((i = tmpStr.find(" ")) >= 0)
		   tmpStr.erase(i,1); 
	    if ((i = tmpStr.find("activateLdap=")) >= 0)
            {
		tmpStr.erase(0,13); 
		_wantLdap = tmpStr;
	    }
	    if ((i = tmpStr.find("ldapHost=")) >= 0)
            {
		tmpStr.erase(0,9); 
		_ldapHostname = tmpStr;
	    }
	    if ((i = tmpStr.find("ldapPort=")) >= 0)
            {
		tmpStr.erase(0,9); 
		_ldapPort = tmpStr;
	    }
	    if ((i = tmpStr.find("ldapBase=")) >= 0)
            {
		tmpStr.erase(0,9); 
		_ldapBase = tmpStr;
	    }
	}
    }
    return true;
}


/*
* created ldap connection with settings from _capiIsdnMonConfig
* settings are changeable in gtkmenu
*/
void capiIsdnMon::initLdap()
{
        try
        {
	    int port = 389;
	    port=atoi(_capiIsdnMonConfig->getLdapPort().c_str());
    	    _ldapAddressServer = new ldapAddressServer(
			    _capiIsdnMonConfig->getLdapHostname(),
			    port,
			    _capiIsdnMonConfig->getLdapBase()
			    );
            capiIsdnMonLogger->addToLog(string("Ldap Connection successfull")) ;
        }
        catch (std::string err_msg)
        {
	    delete _ldapAddressServer;
	    _ldapAddressServer=NULL;
            capiIsdnMonLogger->addToLog(err_msg);
        }
}
