#include "ldapaddressbook.h"
#include <iostream>

/**
* Class constructor.
* initialize ldap stuff:
* anonymous connect to given ldap server
* @param ldapHost hostname from ldap server
* @param ldapPort portnumer from ldap server
*/
ldapAddressServer::ldapAddressServer(std::string ldapHost, int ldapPort, std::string searchBase)
{
    int rc;
    _ldapHost = ldapHost;
    _ldapPort = ldapPort;
    _searchBase = searchBase;
    ld = NULL;
 
    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost.c_str(), ldapPort )) == NULL)
    {
        throw std::string("ldap_init error");
    }
    

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, NULL, NULL );
    if (rc != LDAP_SUCCESS )
    {
        throw std::string(std::string("ldap ldap_simple_bind error: ") + std::string(ldap_err2string( rc )));
    }
}

/**
* Class destructor.
* frees ldap stuff:
* close connection
*/
ldapAddressServer::~ldapAddressServer()
{
   if (ld != NULL)
	   ldap_unbind( ld );  
}

/**
* resolves cn name from phonenumber
* homePhone, mobile , telephoneNumber is searched.
* @param NrWithoutLeadingZero the number to search
* @return "not in addressbook, if not found, else the found cn name
*/
std::string ldapAddressServer::resolveNameForNumber(std::string NrWithoutLeadingZero)
{   
    int         i, rc;
    char        *attribute, **values;
    BerElement  *ber;
    LDAPMessage *searchResult, *entry;
    struct timeval timeOut = {10,0};   /* 10 second connection/search timeout */
    
    

    std::string filterString = std::string("(|(|(homePhone=*") + NrWithoutLeadingZero + std::string(")")
	                         + std::string("(mobile=*")    + NrWithoutLeadingZero + std::string("))")
				 + std::string("(telephoneNumber=*") + NrWithoutLeadingZero + std::string("))");  
	    
	    
    /* Search the directory */
    rc = ldap_search_ext_s(  
                    ld,                    /* LDAP session handle */
                    _searchBase.c_str(),   /* container to search */
                    LDAP_SCOPE_ONELEVEL,   /* search scope */
                    filterString.c_str(),     /* search filter */
                    NULL,                  /* return all attributes */
                    0,                     /* return attributes and values */
                    NULL,                  /* server controls */
                    NULL,                  /* client controls */
                    &timeOut,              /* search timeout */
                    LDAP_NO_LIMIT,         /* no size limit */
                    &searchResult );       /* returned results */
    
    if ( rc != LDAP_SUCCESS ) 
    {
        ldap_msgfree( searchResult );
        //ldap_unbind_s( ld );
        throw (std::string("ldap ldap_search_ext_s error: wrong base?"));
    }   
     
    std::string nameString("not in addressbook");

    /* Go through the search results by checking entries */
    for (   entry   =   ldap_first_entry( ld, searchResult ); 
            entry   !=  NULL; 
            entry   =   ldap_next_entry( ld, entry ) ) 
    {
        for (   attribute = ldap_first_attribute( ld, entry, &ber );
                attribute != NULL; 
                attribute = ldap_next_attribute( ld, entry, ber ) ) 
        {   
            /* Get values and print.  Assumes all values are strings. */
            if (( values = ldap_get_values( ld, entry, attribute)) != NULL ) 
            {
                for ( i = 0; values[i] != NULL; i++ )
		{
		    if (std::string(attribute)==std::string("cn"))
		      nameString=std::string( values[i]);  
		}
                ldap_value_free( values );
            }
            ldap_memfree( attribute );
        }
       
        ber_free(ber, 0);

    }
    ldap_msgfree( searchResult );
    return( nameString );
}
