//$Log: logger.cpp,v $
//Revision 1.3  2005/01/30 23:55:40  tubsmaster
//- log file gets flushed every start
//- check if /dev/capi20 is writeable and readable
//
//Revision 1.2  2004/10/20 18:28:36  tubsmaster
//no more warnings..
//
//Revision 1.1.1.1  2004/06/17 07:17:43  tubsmaster
//initial gtk version
//
//Revision 1.1.1.1  2004/06/05 12:43:28  tubsmaster
//First gtk release
//
//Revision 1.1.1.1  2004/05/21 13:10:42  tubsmaster
//Initial import.
//
//Revision 1.4  2004/05/20 10:39:29  tubsmaster
//logging now works
//
//Revision 1.3  2004/05/20 09:15:38  tubsmaster
//add doxygen doc
//

#include <fstream>
#include "logger.h"
#include <iostream>

static const char logger_cpp[] = "$Id: logger.cpp,v 1.3 2005/01/30 23:55:40 tubsmaster Exp $"; 

using namespace std;


/** 
* Class constructor.
* Sets the LogFile and initializes the mutex.
* @param myLogFile the file (with path), which should be used as Logfile
*/
Logger::Logger (std::string myLogFile)
{
  pthread_mutex_init (&mutexLog, NULL);
  LogFile = myLogFile;
  ofstream filestr;
  filestr.open (LogFile.c_str (), ofstream::trunc);
  filestr.close();
}

/** 
* Class destructor.
* frees mutex.
*/
Logger::~Logger ()
{
  pthread_mutex_destroy (&mutexLog);
}

/**
* adds a line to Logfile.
* This is used to add a line to Log. Its threadsafe, I hope.
* @param stringToLog the String to log (should be without \n to have a correct format in logfile).
* @return returns true if it could add the line to log, else false
*/
bool Logger::addToLog (std::string stringToLog)
{
  time_t ctime;
  time (&ctime);
  struct tm myTm;
  localtime_r (&ctime, &myTm);
  char logline[200];
  sprintf (logline, "%d-%02d-%02d %02d:%02d:%02d      %s",
	   int (myTm.tm_year + 1900), myTm.tm_mon, myTm.tm_mday, myTm.tm_hour,
	   myTm.tm_min, myTm.tm_sec, stringToLog.c_str());

  fstream filestr;

  pthread_mutex_lock (&mutexLog);
  filestr.open (LogFile.c_str (), fstream::out | fstream::app);
  if (!filestr.good ())
    {
#ifdef DEBUG
      std::cout << "Cant open Logfile: " << LogFile << std::endl;
#endif
      filestr.close ();
      pthread_mutex_unlock (&mutexLog);
      return false;
    }
  filestr << logline << std::endl;
  filestr.close ();
  pthread_mutex_unlock (&mutexLog);
  return true;
}
